# -*- coding: utf-8 -*-
'''
    saltpylint.version
    ~~~~~~~~~~~~~~~~~~~

    :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
    :copyright: © 2013 by the SaltStack Team, see AUTHORS for more details.
    :license: Apache 2.0, see LICENSE for more details.
'''

__version_info__ = (2015, 4, 25)
__version__ = '{0[0]}.{0[1]:0>2}.{0[2]:0>2}'.format(__version_info__)
